import 'dart:math';

import 'package:cramr_ui_lib/model/cardinality_selection.dart';
import 'package:cramr_ui_lib/settings/settings_controller.dart';
import 'package:cramr_ui_lib/settings/settings_service.dart';
import 'package:cramr_ui_lib/widgets/cardinality_selection_panel.dart';
import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:provider/provider.dart';

final Finder singularSwitchFinder = find.byKey(const Key('singularToggle'));
final Finder pluralSwitchFinder = find.byKey(const Key('pluralToggle'));

main() {
  testWidgets('check for presence of keys and toggling behaviour',
      (WidgetTester tester) async {
    var cardinalitySelection = CardinalitySelection(true, true);
    await tester.pumpWidget(ChangeNotifierProvider(
        create: (BuildContext context) {
          return cardinalitySelection;
        },
        child: MaterialApp(
          home: CardinalitySelectionPanel(SettingsController(
              SettingsService(), ThemeMode.dark, cardinalitySelection)),
        )));

    expect(singularSwitchFinder, findsOneWidget);
    expect(pluralSwitchFinder, findsOneWidget);
    expect(true, (tester.widget(singularSwitchFinder) as Switch).value);
    expect(true, (tester.widget(pluralSwitchFinder) as Switch).value);
    await tester.tap(singularSwitchFinder);
    await tester.pumpAndSettle();
    expect(false, (tester.widget(singularSwitchFinder) as Switch).value);
    expect(true, (tester.widget(pluralSwitchFinder) as Switch).value);
    await tester.tap(pluralSwitchFinder);
    await tester.pumpAndSettle();
    expect(true, (tester.widget(singularSwitchFinder) as Switch).value);
    expect(false, (tester.widget(pluralSwitchFinder) as Switch).value);
  });

  testWidgets('test the manipulation of the dirty flag in the controller',
      (WidgetTester tester) async {
    var cardinalitySelection = CardinalitySelection(true, true);
    await tester.pumpWidget(ChangeNotifierProvider(
        create: (BuildContext context) {
          return cardinalitySelection;
        },
        child: MaterialApp(
          home: CardinalitySelectionPanel(SettingsController(
              SettingsService(), ThemeMode.dark, cardinalitySelection)),
        )));

    expect(singularSwitchFinder, findsOneWidget);
    expect(pluralSwitchFinder, findsOneWidget);
    expect(true, (tester.widget(singularSwitchFinder) as Switch).value);
    expect(true, (tester.widget(pluralSwitchFinder) as Switch).value);
    await tester.tap(singularSwitchFinder);
    await tester.pumpAndSettle();
    expect(true, cardinalitySelection.isDirty);
    await tester.tap(singularSwitchFinder);
    await tester.pumpAndSettle();
    expect(false, cardinalitySelection.isDirty);
  });
}
